@@ -5,7 +5,7 @@ package Unix::Mgt;
 use strict;
 use IPC::System::Simple 'runx';
 use Capture::Tiny 'capture_merged';
-use String::Util qw{define nocontent};
+use String::Util qw{define nocontent rtrim};
 use Unix::SearchPathGuess 'cmd_path_guess';
 use Carp 'croak';
 
@@ -14,7 +14,7 @@ use Carp 'croak';
 # use Debug::ShowStuff::ShowVar;
 
 # version
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 
 #------------------------------------------------------------------------------
@@ -22,7 +22,7 @@ our $VERSION = '0.13';
 #
 use base 'Exporter';
 use vars qw[@EXPORT_OK %EXPORT_TAGS];
-push @EXPORT_OK, 'unix_mgt_err';
+push @EXPORT_OK, qw{unix_mgt_err unix_mgt_err_id};
 %EXPORT_TAGS = ('all' => [@EXPORT_OK]);
 #
 # export
@@ -85,40 +85,6 @@ of Unix::Mgt before it has all the features that might be expected. This
 version does not include methods for removing users from groups, renaming
 users or groups, or several other methods.
 
-=head2 Help with BSD development
-
-This version does not work well on BSDish systems. Although the methods for
-retrieving information such as C<get> and C<group> work well, this module
-currently cannot create, modify, or delete users or group. If you'd like to
-help fill in these features, here's what needs to be done.
-
-Several places in the code is the comment "BSD code needed". In those places
-what we need is to build an array consisting of an external command and the
-arguments to be sent to the command. It would look something like this:
-
- # build command
- @cmd = (
-    'pw', 
-    'useradd', 
-    $user->{'name'},
-    '-G', 
-    $group->{'name'},
- );
-
-The first element of the array is the program to be run. You don't need to put
-the full path of the program, L<Unix::SearchPathGuess> should find it. (If it
-doesn't please let me know, we may need to fix Unix::SearchPathGuess.) The
-remaining elements are the params sent to the program using
-L<IPC::System::Simple>. The program is not run in a shell, but instead is
-called directly, so it isn't necessary to do any shell escaping.
-
-The following methods currently need patching. In Unix::Mgt::User we need
-create(), field_get_set(), group(), add_to_group(), and remove(). In
-Unix::Mgt::Group we need create() and remove(). We will need more as we add
-features to this module.
-
-Please email me at miko@idocs.com if you've made these mods.
-
 =cut
 
 #
@@ -153,7 +119,14 @@ sub reset_err {
 
 sub unix_mgt_err {
 	if ($err_id)
-		{ return $err_id . ': ' . define($err_msg) . "\n" }
+		{ return $err_id . rtrim(': ' . define($err_msg))}
+	else
+		{ return '' }
+}
+
+sub unix_mgt_err_id {
+	if ($err_id)
+		{ return $err_id}
 	else
 		{ return '' }
 }
@@ -251,7 +224,7 @@ sub run_cmd {
 #------------------------------------------------------------------------------
 # use_bsd
 #
-my ($use_bsd);
+our ($use_bsd);
 
 sub use_bsd {
 	my ($ug) = @_;
@@ -310,12 +283,17 @@ use overload
 sub normalize_name {
 	my ($class, $name) = @_;
 	
-	# if defined, remove spaces at beginning and end
+	# TESTING
+	# println subname(method=>1), ' $name: ', $name; ##i
+	
+	# if defined, remove eerything after first space
 	if (defined $name) {
-		$name =~ s|^\s+||sg;
-		$name =~ s|\s+$||sg;
+		$name =~ s|\s.*||sg;
 	}
 	
+	# TESTING
+	# showvar $name;
+	
 	# return
 	return $name;
 }
@@ -444,6 +422,8 @@ package Unix::Mgt::User;
 use strict;
 use Carp 'croak';
 use String::Util ':all';
+use Unix::SearchPathGuess 'cmd_path_guess';
+use IPC::System::Simple 'capturex';
 use base 'Unix::Mgt::UGCommon';
 
 # debug tools
@@ -461,8 +441,8 @@ our $MOD_ONLY;
 
 A Unix::Mgt::User object represents a user in the Unix system. The object
 allows you to get and set information about the user account. A user object
-is created in one of three ways: C<get>, C<create>, or C<ensure>. Note that
-there is no C<new> method.
+is created in one of three ways: C<get>, C<create>, or C<ensure>. The C<new>
+method is an alias for C<get>.
 
 Unix::Mgt::User objects stringify to the account's name. For example, the
 following code would output C<miko>.
@@ -516,6 +496,12 @@ C<$Unix::Mgt::err_id> and undef is returned.
 
 =cut
 
+# alias new to get
+sub new {
+	my $class = shift(@_);
+	return $class->get(@_);
+}
+
 sub get {
 	my $class = shift(@_);
 	my ($name, %opts) = $class->even_odd_params('name', @_);
@@ -659,17 +645,17 @@ sub create {
 	
 	# BSD style
 	if ($class->use_bsd) {
-		# BSD code needed
-		
-		croak
-			'We have not yet implemented for BSD. ' .
-			'Please see "Help with BSD development" in the Unix::Mgt ' .
-			'documents to help with this issue.';
+		@cmd = (
+			'pw',
+			'useradd',
+			'-n',
+			'user-1',
+			'-m'
+		);
 	}
 	
 	# else Linux style
 	else {
-		# build command
 		@cmd = (
 			'adduser',
 			'--disabled-password',
@@ -816,6 +802,12 @@ Called with a single param, shell() sets, then returns the user's shell:
 
 =cut
 
+our %bsd_switches = (
+	home => 'd',
+	gid => 'g',
+	shell => 's',
+);
+
 sub field_get_set {
 	my $user = shift(@_);
 	my $field = shift(@_);
@@ -834,12 +826,13 @@ sub field_get_set {
 		
 		# BSD style
 		if ($user->use_bsd) {
-			# BSD code needed
-			
-			croak
-				'We have not yet implemented for BSD. ' .
-				'Please see "Help with BSD development" in the Unix::Mgt ' .
-				'documents to help with this issue.';
+			@cmd = (
+				'pw',
+				'usermod',
+				$user->{'name'},
+				'-' . $bsd_switches{$option},
+				$value
+			);
 		}
 		
 		# else Linux style
@@ -853,6 +846,10 @@ sub field_get_set {
 			);
 		}
 		
+		# TESTING
+		# sudo pw usermod -d "/home/whatever"
+		# showaref @cmd;
+		
 		# run command
 		$user->run_cmd("usermod-error-$field", @cmd) or return undef;
 	}
@@ -915,12 +912,13 @@ sub group {
 		
 		# BSD style
 		if ($user->use_bsd) {
-			# BSD code needed
-			
-			croak
-				'We have not yet implemented for BSD. ' .
-				'Please see "Help with BSD development" in the Unix::Mgt ' .
-				'documents to help with this issue.';
+			@args = (
+				'pw',
+				'usermod',
+				$user->{'name'},
+				'-g',
+				"$new_group",
+			);
 		}
 		
 		# else Linux style
@@ -1075,12 +1073,13 @@ sub add_to_group {
 	
 	# BSD style
 	if ($user->use_bsd) {
-		# BSD code needed
-		
-		croak
-			'We have not yet implemented for BSD. ' .
-			'Please see "Help with BSD development" in the Unix::Mgt ' .
-			'documents to help with this issue.';
+		@args = (
+			'pw',
+			'usermod',
+			$user->{'name'},
+			'-G',
+			"$group"
+		);
 	}
 	
 	# else Linux style
@@ -1121,6 +1120,7 @@ parameters.
 
 sub remove {
 	my ($user) = @_;
+	my (@args);
 	
 	# TESTING
 	# println subname(method=>1); ##i
@@ -1133,19 +1133,25 @@ sub remove {
 	
 	# BSD style
 	if ($user->use_bsd) {
-		# BSD code needed
-		
-		croak
-			'We have not yet implemented for BSD. ' .
-			'Please see "Help with BSD development" in the Unix::Mgt ' .
-			'documents to help with this issue.';
+		@args = (
+			'pw',
+			'userdel',
+			'-n',
+			$user->name,
+			'-r',
+		);
 	}
 	
 	# else use userdel
 	else {
-		$user->run_cmd('error-deleting-user', 'userdel', $user->name) or return undef;
+		@args = (
+			'userdel',
+			$user->name
+		);
 	}
 	
+	$user->run_cmd('error-deleting-user', @args) or return undef;
+	
 	# return
 	return 1;
 }
@@ -1154,6 +1160,254 @@ sub remove {
 #------------------------------------------------------------------------------
 
 
+#------------------------------------------------------------------------------
+# default_inc
+#
+sub default_inc {
+	my ($env_path, $perl_path, $default_str, @default_arr);
+	
+	# TESTING
+	# println subname(); ##i
+	
+	# path to perl binary currently being run
+	$perl_path = $^X;
+	
+	# path to the env command
+	$env_path = cmd_path_guess('env');
+	$env_path or return undef;
+	
+	# get raw string containing the default @INC
+	eval {
+		$default_str = capturex(
+			$env_path,
+			'-i',
+			$perl_path,
+			'-wT',
+			'-e',
+			'print join(":", @INC)',
+		);
+	};
+	
+	# if the capture threw an error, return empty array
+	if ($@) {
+		# println 'error getting default @INC: ', $@;
+		return ();
+	}
+	
+	# if we didn't get a defined string, return empty array
+	if (! defined $default_str)
+		{ return () }
+	
+	# parse string
+	@default_arr = split(':', $default_str);
+	
+	# return
+	return (@default_arr);
+}
+#
+# default_inc
+#------------------------------------------------------------------------------
+
+
+#------------------------------------------------------------------------------
+# new_libs
+# private sub
+#
+sub new_libs {
+	my (@defaults_arr, %defaults, @rv);
+	
+	# TESTING
+	# println subname(as_sub=>1); ##i
+	
+	# get defaults, return if none
+	@defaults_arr = default_inc();
+	@defaults_arr or return();
+	
+	# build hash of defaults
+	@defaults{@defaults_arr} = ();
+	
+	# loop through @INC, adding paths that aren't in defaults
+	LIB_LOOP:
+	foreach my $lib (@INC) {
+		# special case: remove current directory
+		if ($lib eq '.')
+			{ next LIB_LOOP }
+		
+		# if the library isn't in the defaults, add it to the return array
+		if (! exists $defaults{$lib})
+			{ push @rv, $lib }
+	}
+	
+	# return
+	return join(':', @rv);
+}
+#
+# new_libs
+#------------------------------------------------------------------------------
+
+
+#------------------------------------------------------------------------------
+# file_tests
+# TODO:
+#	- Write regtests for this method.
+#	- Document this method.
+#
+# Document: If -e returns false, not other tests are run. -e does not indiciate
+# why it returns false. I might be because the file doesn't exist. It also
+# might be that the user doesn't have permission to even see the file.
+#
+# Regtest: test with multiple extra libs
+#
+sub file_tests {
+	my ($user, $paths_in) = @_;
+	my (@cmd, $single, @paths, $cmd_str, @su, $json, $tests);
+	
+	# TESTING
+	# println subname(); ##i
+	
+	# load JSON module
+	require JSON::Tiny;
+	
+	# paths must be sent
+	if (! defined $paths_in)
+		{ croak 'defined $paths_in not sent' }
+	
+	# build @paths
+	if (ref $paths_in) {
+		@paths = @$paths_in;
+	}
+	else {
+		@paths = $paths_in;
+		$single = $paths[0];
+	}
+	
+	# remove empties
+	@paths = grep {$_ =~ m|\S|s} @paths;
+	
+	# if no paths, nothing to do
+	if (! @paths)
+		{ croak 'no @paths sent' }
+	
+	# escape quotes, add quotes
+	foreach my $path (@paths) {
+		$path =~ s|\"|\\"|sg;
+		$path =~ s|\'|\\'|sg;
+		$path = "'$path'";
+	}
+	
+	# path to perl binary currently being run
+	push @cmd, $^X, '-wT';
+	
+	# extra libs to add to perl command
+	if (my $new_libs = new_libs()) {
+		my @libs = split(':', $new_libs);
+		
+		# loop through libs adding each one to command
+		foreach my $lib (@libs) {
+			push @cmd, '-I', $lib;
+		}
+	}
+	
+	# add Unix::Mgt module
+	push @cmd, '-MUnix::Mgt';
+	
+	# add call to file_tests_from_external
+	push @cmd, '-e', '"Unix::Mgt::User::file_tests_from_external(' . join(', ', @paths) . ')"';
+	
+	# get command to send to su
+	$cmd_str = join(' ', @cmd);
+	
+	# build su command
+	@su = (
+		cmd_path_guess('su'),
+		$user->name(),
+		'-c',
+		$cmd_str,
+	);
+	
+	# run command, get results
+	$json = capturex(@su);
+	
+	# decode json
+	eval {
+		$tests = JSON::Tiny::decode_json($json);
+	};
+	
+	# throw error if unable to decode
+	if (! $tests) {
+		croak 'unable to parse results from file tests';
+	}
+	
+	# if single, just return the tests for that file
+	if ($single)
+		{ return $tests->{$single} }
+	
+	# else return the entire hash
+	else
+		{ return $tests }
+}
+#
+# file_tests
+#------------------------------------------------------------------------------
+
+
+#------------------------------------------------------------------------------
+# file_tests_from_external
+# Don't call this sub directly.
+#
+sub file_tests_from_external {
+	my (@paths) = @_;
+	my ($rv);
+	
+	# TESTING
+	# println subname(); ##i
+	
+	# load JSON module
+	require JSON::Tiny;
+	
+	# initialize return hashref
+	$rv = {};
+	
+	# loop through paths
+	PATH_LOOP:
+	foreach my $path (@paths) {
+		my ($tests);
+		
+		# TESTING
+		# println '$path: ', $path; ##i
+		
+		# initialize tests for this path
+		$rv->{$path} = $tests = {};
+		
+		# -e file exists
+		unless ($tests->{'-e'} = -e($path)) {
+			next PATH_LOOP;
+		}
+		
+		# add rest of tests
+		$tests->{'-r'} = -r($path);   # -r  File is readable by effective uid/gid.
+		$tests->{'-w'} = -w($path);   # -w  File is writable by effective uid/gid.
+		$tests->{'-x'} = -x($path);   # -x  File is executable by effective uid/gid.
+		$tests->{'-o'} = -o($path);   # -o  File is owned by effective uid.
+		$tests->{'-R'} = -R($path);   # -R  File is readable by real uid/gid.
+		$tests->{'-W'} = -W($path);   # -W  File is writable by real uid/gid.
+		$tests->{'-X'} = -X($path);   # -X  File is executable by real uid/gid.
+		$tests->{'-O'} = -O($path);   # -O  File is owned by real uid.
+	}
+	
+	# TODO:
+	#	- test for multiple files
+	#	- write tests
+	#	- document
+	
+	# output
+	print JSON::Tiny::encode_json($rv);
+}
+#
+# file_tests_from_external
+#------------------------------------------------------------------------------
+
+
 
 #
 # Unix::Mgt::User
@@ -1185,8 +1439,8 @@ our $MOD_ONLY;
 
 A Unix::Mgt::Group object represents a group in the Unix system. The object
 allows you to get and set information about the group. A group object is
-created in one of three ways: C<get>, C<create>, or C<ensure>. Note that
-there is no C<new> method.
+created in one of three ways: C<get>, C<create>, or C<ensure>. The C<new>
+method is an alias for C<get>.
 
 Unix::Mgt::Group objects stringify to the groups's name. For example, the
 following code would output C<video>.
@@ -1253,6 +1507,12 @@ C<$Unix::Mgt::err_id> and undef is returned.
 
 =cut
 
+# alias new to get
+sub new {
+	my $class = shift(@_);
+	return $class->get(@_);
+}
+
 sub get {
 	my $class = shift(@_);
 	my ($name, %opts) = $class->even_odd_params('name', @_);
@@ -1339,12 +1599,12 @@ sub create {
 	
 	# BSD style
 	if ($class->use_bsd) {
-		# BSD code needed
-		
-		croak
-			'We have not yet implemented for BSD. ' .
-			'Please see "Help with BSD development" in the Unix::Mgt ' .
-			'documents to help with this issue.';
+		@cmd = (
+			'pw',
+			'groupadd',
+			'-n',
+			$name,
+		);
 	}
 	
 	# else Linux style
@@ -1681,6 +1941,7 @@ If any users have the group as a primary group then this method will fail.
 
 sub remove {
 	my ($group) = @_;
+	my (@cmd);
 	
 	# TESTING
 	# println subname(method=>1); ##i
@@ -1719,19 +1980,25 @@ sub remove {
 	
 	# if using pw command
 	if ($group->use_bsd) {
-		# BSD code needed
-		
-		croak
-			'We have not yet implemented for BSD. ' .
-			'Please see "Help with BSD development" in the Unix::Mgt ' .
-			'documents to help with this issue.';
+		@cmd = (
+			'pw',
+			'groupdel',
+			'-n',
+			$group->name,
+		);
 	}
 	
 	# else use groupdel
 	else {
-		$group->run_cmd('error-deleting-group', 'groupdel', $group->name) or return undef;
+		@cmd = (
+			'groupdel',
+			$group->name,
+		);
 	}
 	
+	# run command
+	$group->run_cmd('error-deleting-group', @cmd) or return undef;
+	
 	# return
 	return 1;
 }
@@ -1802,6 +2069,13 @@ Added $user->remove() and $group->remove().
 Added slots where BSD-style commands will go. Currently, methods for creating,
 modifying, or deleting users or group will fail on BSD.
 
+=item Version 0.14 February 2, 2015
+
+Added support for BSD. The support is poorly tested because I don't have a BSD
+system. Any feedback is appreciated.
+
+Added Unix::Mgt::User::new and Unix::Mgt::Group::new as aliases to get().
+
 =back
 
 =cut
