@@ -1,6 +1,7 @@
 use strict;
 use IPC::System::Simple 'capturex';
 use Unix::SearchPathGuess 'cmd_path_guess';
+use Carp 'confess';
 
 # debugging
 # use Debug::ShowStuff ':all';
@@ -11,12 +12,13 @@ use Unix::SearchPathGuess 'cmd_path_guess';
 # mgt_err
 #
 sub mgt_err {
-	my ($should, $test_name) = @_;
+	my ($test_name, $should) = @_;
 	
 	# $test_name is required
 	$test_name or confess ('$test_name is required');
 	
-	return comp($Unix::Mgt::err_id, $should, $test_name);
+	# compare
+	return comp($test_name, $Unix::Mgt::err_id, $should);
 }
 #
 # mgt_err
@@ -71,7 +73,7 @@ sub reset_users_groups {
 # This sub is used for the development of tests, not the tests themselves.
 #
 sub show_err {
-	my ($should) = @_;
+	my (%opts) = @_;
 	
 	# if any error, display it
 	if ($Unix::Mgt::err_id) {
@@ -79,11 +81,18 @@ sub show_err {
 			'error:', "\n",
 			"   $Unix::Mgt::err_id\n",
 			"   $Unix::Mgt::err_msg\n";
+		
+		# auto die if called to do so
+		if ($opts{'auto_die'}) {
+			exit 1;
+		}
 	}
 	
 	# else output that there isn't an error
 	else {
-		print "no Unix::Mgt error\n";
+		unless ($opts{'errors_only'}) {
+			print "no Unix::Mgt error\n";
+		}
 	}
 }
 #
