@@ -11,10 +11,12 @@ use Carp 'confess';
 # comp
 #
 sub comp {
-	my ($is, $shouldbe, $test_name) = @_;
+	my ($test_name, $is, $shouldbe) = @_;
 	
 	# TESTING
 	# println subname(); ##i
+	# showvar $is;
+	# showvar $shouldbe;
 	
 	# $test_name is required
 	$test_name or confess ('$test_name is required');
@@ -27,11 +29,13 @@ sub comp {
 				"\tshould be : ", (defined($shouldbe) ? $shouldbe : '[undef]'), "\n\n";
 		}
 		
-		set_ok(0, "$test_name: values do not match");
+		rtok("(rt) $test_name: values do not match", 0);
+		return 0;
 	}
 	
 	else {
-		set_ok(1, "$test_name: values match");
+		rtok("(rt) $test_name: values match", 1);
+		return 1;
 	}
 }
 #
@@ -75,7 +79,7 @@ sub arr_comp {
 	
 	# if they have different lengths, they're different
 	if (@$alpha_sent != @$beta_sent)
-		{ set_ok(0, $test_name) }
+		{ rtok("(rt) $test_name", 0) }
 	
 	# get arrays to use for comparison
 	@alpha = @$alpha_sent;
@@ -100,7 +104,7 @@ sub arr_comp {
 			( (  defined $alpha[$i]) && (! defined $beta[$i]) ) ||
 			( (! defined $alpha[$i]) && (  defined $beta[$i]) )
 			) {
-			set_ok(0, $test_name);
+			rtok("(rt) $test_name", 0);
 		}
 		
 		# if $alpha[$i] is undef then both must be, so they're the same
@@ -110,12 +114,12 @@ sub arr_comp {
 		# both are defined
 		else {
 			unless ($alpha[$i] eq $beta[$i])
-				{ set_ok(0, $test_name) }
+				{ rtok("(rt) $test_name", 0) }
 		}
 	}
 	
 	# if we get this far, they're the same
-	set_ok(1, $test_name);
+	rtok("(rt) $test_name", 1);
 }
 #
 # arr_comp
@@ -123,18 +127,29 @@ sub arr_comp {
 
 
 #------------------------------------------------------------------------------
-# check_isa
+# file_exists
 #
-sub check_isa {
-	my ($ob, $class, $test_name) = @_;
+sub file_exists {
+	my ($test_name, $path, $should) = @_;
+	my ($e);
+	
+	# TESTING
+	# println subname(); ##i
 	
 	# $test_name is required
 	$test_name or confess ('$test_name is required');
 	
-	set_ok(UNIVERSAL::isa( $ob, $class), $test_name);
+	# test if the file exists
+	$e = -e($path);
+	
+	# if not should, reverse sense of test
+	if (! $should)
+		{ $e = ! $e }
+	
+	rtok("(rt) $test_name", $e);
 }
 #
-# check_isa
+# file_exists
 #------------------------------------------------------------------------------
 
 
@@ -185,38 +200,6 @@ sub stringify_tokens {
 #------------------------------------------------------------------------------
 
 
-#------------------------------------------------------------------------------
-# set_ok
-#
-sub set_ok {
-	my ($ok, $test_name) = @_;
-	
-	# TESTING
-	# println subname(); ##i
-	
-	# $test_name is required
-	$test_name or confess ('$test_name is required');
-	
-	# development environment
-	if ($ENV{'IDOCSDEV'}) {
-		if ($ok) {
-			return 1;
-		}
-		
-		else {
-			die($test_name);
-		}
-	}
-	
-	# else regular ok
-	# ok($ok, '[1] ' . compact($test_name));
-	ok($ok, compact($test_name));
-}
-#
-# set_ok
-#------------------------------------------------------------------------------
-
-
 #------------------------------------------------------------------------------
 # key_count
 #
@@ -227,7 +210,8 @@ sub key_count {
 	$test_name or confess ('$test_name is required');
 	
 	unless (scalar(keys %$hash) == $count) {
-		set_ok(
+		rtok(
+			"(rt) $test_name",
 			0,
 			
 			'hash should have ' .
@@ -236,12 +220,10 @@ sub key_count {
 			( ($count == 1) ? '' : 's' ) . ' ' .
 			'but actually has ' .
 			scalar(keys %$hash),
-			
-			$test_name
 		);
 	}
 	
-	set_ok(1, $test_name);
+	rtok("(rt) $test_name", 1);
 }
 #
 # key_count
@@ -252,7 +234,7 @@ sub key_count {
 # bool_check
 #
 sub bool_check {
-	my ($is_got, $should_got, $test_name) = @_;
+	my ($test_name, $is_got, $should_got) = @_;
 	my ($is_norm, $should_norm);
 	
 	# $test_name is required
@@ -267,14 +249,14 @@ sub bool_check {
 			"\n",
 			"\tis:         ", (defined($is_got)     ? $is_got     : '[undef]'), "\n",
 			"\tshould be : ", (defined($should_got) ? $should_got : '[undef]'), "\n\n";
-		set_ok(0, $test_name);
+		rtok("(rt) $test_name", 0);
 		
 		# return false
 		return 0;
 	}
 	
 	# ok
-	set_ok(1, $test_name);
+	rtok("(rt) $test_name", 1);
 	
 	# return true
 	return 1;
@@ -295,7 +277,8 @@ sub el_count {
 	$test_name or confess ('$test_name is required');
 	
 	unless (scalar(@$arr) == $count) {
-		set_ok(
+		rtok(
+			"(rt) $test_name",
 			0,
 			
 			'array should have ' .
@@ -304,12 +287,10 @@ sub el_count {
 			( ($count == 1) ? '' : 's' ) . ' ' .
 			'but actually has ' .
 			scalar(@$arr),
-			
-			$test_name
 		);
 	}
 	
-	set_ok(1, $test_name);
+	rtok("(rt) $test_name", 1);
 }
 #
 # el_count
@@ -328,6 +309,7 @@ sub compact {
 		$val =~ s|\s+| |sg;
 	}
 	
+	# return
 	return $val;
 }
 #
@@ -335,5 +317,99 @@ sub compact {
 #------------------------------------------------------------------------------
 
 
+#------------------------------------------------------------------------------
+# error_id_check
+#
+sub error_id_check {
+	my ($test_name, $is, $should) = @_;
+	
+	# TESTING
+	# println subname(); ##i
+	# showvar $is;
+	# showvar $should;
+	
+	# $test_name is required
+	$test_name or confess ('$test_name is required');
+	
+	# get id of $is
+	if (defined $is)
+		{ $is =~ s|\:.*||s }
+	
+	# get id of $should or set it to empty string
+	if (defined $should)
+		{ $should =~ s|\:.*||s }
+	else
+		{ $should = '' }
+	
+	# compare
+	comp($test_name, $is, $should);
+}
+#
+# error_id_check
+#------------------------------------------------------------------------------
+
+
+#------------------------------------------------------------------------------
+# randstring
+#
+sub randstring {
+	my ($rv, $count, @chars);
+	$rv = '';
+	$count = 5;
+	
+	# set of characters to choose from
+	# Only using alphas because numbers often cause problems as names for
+	# various objects like database tables.
+	@chars = ('a' .. 'z', 'A' .. 'Z');
+	
+	while (length($rv) < $count) {
+		my $char = rand();
+		$char = int( $char * ($#chars + 1) );
+		$char = $chars[$char];
+		$rv .= $char;
+	}
+	
+	# return
+	return $rv;
+}
+#
+# randstring
+#------------------------------------------------------------------------------
+
+
+#------------------------------------------------------------------------------
+# taint
+#
+sub taint {
+	my ($val) = @_;
+	
+	# add tainted empty string to value
+	if (defined $val) {
+		my $tainted = <*>;
+		$tainted =~ s|.*||g;
+		$val .= $tainted;
+	}
+	
+	# return
+	return $val;
+}
+#
+# taint
+#------------------------------------------------------------------------------
+
+
+
+#------------------------------------------------------------------------------
+# rtok
+#
+sub rtok {
+	my ($test_name, $ok) = @_;
+	ok($ok, $test_name);
+}
+#
+# rtok
+#------------------------------------------------------------------------------
+
+
 # return true
 1;
